@@ -6,14 +6,12 @@ use Text::Xslate::Util qw(
     value_to_literal
 );
 
-
 use Carp ();
 use Scalar::Util ();
 
 use Text::Xslate::Util qw(
     $DEBUG p neat
     value_to_literal
-    mark_raw unmark_raw html_escape
 );
 use Text::Xslate::PP::Const;
 use Text::Xslate::PP::Booster;
@@ -65,7 +63,6 @@ sub opcode_to_perlcode {
     my ( $self, $opcode ) = @_;
 
     my $perlcode = $self->opcode_to_perlcode_string( $opcode );
-    print STDERR $perlcode, "\n" if _DUMP_PP;
     return Text::Xslate::PP::Booster->compile($perlcode);
 }
 
@@ -99,6 +96,7 @@ CODE
     $perlcode .= join( '', grep { defined } @{ $self->{lines} } );
     $perlcode .= "\n" . '    return $output;' . "\n}";
 
+    print STDERR $perlcode, "\n" if _DUMP_PP;
     return $perlcode;
 }
 
@@ -504,7 +502,17 @@ $CODE_MANIP{ 'builtin_unmark_raw' } = sub  {
 
 $CODE_MANIP{ 'builtin_html_escape' } = sub  {
     my ( $self, $arg, $line ) = @_;
-    $self->sa( sprintf( 'Text::Xslate::html_escape( %s )', $self->sa ) );
+    $self->sa( sprintf( 'html_escape( %s )', $self->sa ) );
+};
+
+$CODE_MANIP{ 'builtin_uri' } = sub  {
+    my ( $self, $arg, $line ) = @_;
+    $self->sa( sprintf( 'uri_escape( %s )', $self->sa ) );
+};
+
+$CODE_MANIP{ 'builtin_ref' } = sub  {
+    my ( $self, $arg, $line ) = @_;
+    $self->sa( sprintf( 'ref( %s )', $self->sa ) );
 };
 
 $CODE_MANIP{ 'match' } = sub {
@@ -561,6 +569,11 @@ $CODE_MANIP{ 'scmp' } = sub {
     $self->sa( sprintf( '( %s cmp %s )', _rm_tailed_lf( $self->sb() ), _rm_tailed_lf( $self->sa() ) ) );
 };
 
+$CODE_MANIP{ 'range' } = sub {
+    my ( $self, $arg, $line ) = @_;
+    push @{ $self->SP->[ -1 ] }, sprintf('( %s .. %s )',
+        _rm_tailed_lf( $self->sb() ), _rm_tailed_lf( $self->sa() ) );
+};
 
 $CODE_MANIP{ 'macrocall' } = sub {
     my ( $self, $arg, $line ) = @_;
